package cn.com.qmth.stmms.biz.report.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import cn.com.qmth.stmms.common.annotation.ExcelField;

/**
 * 课程总量分析
 *
 * @author xuwenjin
 * @date 2019-04-11
 */
@Entity
@Table(name = "s_basic_subject", indexes = { @Index(name = "subject_index", columnList = "exam_id,subject_code") })
public class ReportSubject implements Serializable {

    private static final long serialVersionUID = -1596478079357284914L;

    // 数据库中多个单元紧凑保存的拼接符
    public static final String DB_ITEM_JOINER = ",";

    /**
     * 主键
     */
    @Id
    @GeneratedValue
    private Integer id;

    /**
     * 考试id
     */
    @Column(name = "exam_id")
    private Integer examId;

    /**
     * 课程编号
     */
    @ExcelField(title = "课程代码", align = 2, sort = 10)
    @Column(name = "subject_code")
    private String subjectCode;

    /**
     * 课程名称
     */
    @ExcelField(title = "课程名称", align = 2, sort = 20)
    @Column(name = "subject_name")
    private String subjectName;

    /**
     * 最高分
     */
    @ExcelField(title = "最高分", align = 2, sort = 30)
    @Column(name = "max_score")
    private Double maxScore;

    /**
     * 最低分
     */
    @ExcelField(title = "最低分", align = 2, sort = 40)
    @Column(name = "min_score")
    private Double minScore;

    /**
     * 平均分
     */
    @ExcelField(title = "平均分", align = 2, sort = 50)
    @Column(name = "avg_score")
    private Double avgScore;

    /**
     * 报考人数
     */
    @ExcelField(title = "报考人数", align = 2, sort = 60)
    @Column(name = "total_count")
    private Integer totalCount;

    /**
     * 及格人数
     */
    @ExcelField(title = "及格人数", align = 2, sort = 70)
    @Column(name = "pass_count")
    private Integer passCount;

    /**
     * 及格率
     */
    @ExcelField(title = "及格率(%)", align = 2, sort = 80)
    @Column(name = "pass_rate")
    private Double passRate;

    /**
     * 优秀率
     */
    @ExcelField(title = "优秀率(%)", align = 2, sort = 90)
    @Column(name = "excellent_rate")
    private Double excellentRate;

    /**
     * 优秀人数
     */
    @ExcelField(title = "优秀人数", align = 2, sort = 100)
    @Column(name = "excellent_count")
    private Integer excellentCount;

    /**
     * 缺考人数
     */
    @ExcelField(title = "缺考人数", align = 2, sort = 110)
    @Column(name = "absent_count")
    private Integer absentCount;

    /**
     * 违纪人数
     */
    @ExcelField(title = "违纪人数", align = 2, sort = 120)
    @Column(name = "breach_count")
    private Integer breachCount;

    /**
     * 有效人数
     */
    @ExcelField(title = "有效人数", align = 2, sort = 130)
    @Column(name = "reality_count")
    private Integer realityCount;

    /**
     * 高低分段统计
     */
    @Column(name = "range_level", nullable = true)
    private String rangeLevel;

    /**
     * 所有选项
     */
    @Column(name = "options", nullable = true)
    private String options;

    /**
     * 难度分布
     */
    @Column(name = "difficulity_level", nullable = true)
    private String difficulityLevel;

    /**
     * 区分度分布
     */
    @Column(name = "discrimination_level", nullable = true)
    private String discriminationLevel;

    /**
     * 一分一段分段统计
     */
    @Column(name = "score_range", nullable = true, columnDefinition = "text")
    private String scoreRange;

    /**
     * 标准差
     */
    @Column(name = "stdev")
    private Double stdev;

    /**
     * 差异系数
     */
    @Column(name = "coefficient", nullable = true, length = 32)
    private Double coefficient;

    /**
     * 难度
     */
    @Column(name = "difficulty", nullable = true, length = 32)
    private Double difficulty;

    /**
     * 区分度
     */
    @Column(name = "discrimination", nullable = true, length = 32)
    private Double discrimination;

    /**
     * 零分人数
     */
    @Column(name = "zero_count", nullable = true, length = 32)
    private Integer zeroCount;

    /**
     * 满分人数
     */
    @Column(name = "full_count")
    private Integer fullCount;

    /**
     * 满分
     */
    @Column(name = "total_score")
    private Double totalScore;

    /**
     * 题目数量
     */
    @Column(name = "question_count")
    private Integer questionCount;

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Integer getExamId() {
        return examId;
    }

    public void setExamId(Integer examId) {
        this.examId = examId;
    }

    public String getSubjectCode() {
        return subjectCode;
    }

    public void setSubjectCode(String subjectCode) {
        this.subjectCode = subjectCode;
    }

    public String getSubjectName() {
        return subjectName;
    }

    public void setSubjectName(String subjectName) {
        this.subjectName = subjectName;
    }

    public Double getMaxScore() {
        return maxScore;
    }

    public void setMaxScore(Double maxScore) {
        this.maxScore = maxScore;
    }

    public Double getMinScore() {
        return minScore;
    }

    public void setMinScore(Double minScore) {
        this.minScore = minScore;
    }

    public Double getAvgScore() {
        return avgScore;
    }

    public void setAvgScore(Double avgScore) {
        this.avgScore = avgScore;
    }

    public Integer getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(Integer totalCount) {
        this.totalCount = totalCount;
    }

    public Integer getPassCount() {
        return passCount;
    }

    public void setPassCount(Integer passCount) {
        this.passCount = passCount;
    }

    public Double getPassRate() {
        return passRate;
    }

    public void setPassRate(Double passRate) {
        this.passRate = passRate;
    }

    public Integer getAbsentCount() {
        return absentCount;
    }

    public void setAbsentCount(Integer absentCount) {
        this.absentCount = absentCount;
    }

    public Integer getBreachCount() {
        return breachCount;
    }

    public void setBreachCount(Integer breachCount) {
        this.breachCount = breachCount;
    }

    public Integer getRealityCount() {
        return realityCount;
    }

    public void setRealityCount(Integer realityCount) {
        this.realityCount = realityCount;
    }

    public String getRangeLevel() {
        return rangeLevel;
    }

    public void setRangeLevel(String rangeLevel) {
        this.rangeLevel = rangeLevel;
    }

    public String getOptions() {
        return options;
    }

    public void setOptions(String options) {
        this.options = options;
    }

    public String getDifficulityLevel() {
        return difficulityLevel;
    }

    public void setDifficulityLevel(String difficulityLevel) {
        this.difficulityLevel = difficulityLevel;
    }

    public String getDiscriminationLevel() {
        return discriminationLevel;
    }

    public void setDiscriminationLevel(String discriminationLevel) {
        this.discriminationLevel = discriminationLevel;
    }

    public String getScoreRange() {
        return scoreRange;
    }

    public void setScoreRange(String scoreRange) {
        this.scoreRange = scoreRange;
    }

    public Double getStdev() {
        return stdev;
    }

    public void setStdev(Double stdev) {
        this.stdev = stdev;
    }

    public Double getCoefficient() {
        return coefficient;
    }

    public void setCoefficient(Double coefficient) {
        this.coefficient = coefficient;
    }

    public Double getDifficulty() {
        return difficulty;
    }

    public void setDifficulty(Double difficulty) {
        this.difficulty = difficulty;
    }

    public Double getDiscrimination() {
        return discrimination;
    }

    public void setDiscrimination(Double discrimination) {
        this.discrimination = discrimination;
    }

    public Integer getZeroCount() {
        return zeroCount;
    }

    public void setZeroCount(Integer zeroCount) {
        this.zeroCount = zeroCount;
    }

    public Integer getFullCount() {
        return fullCount;
    }

    public void setFullCount(Integer fullCount) {
        this.fullCount = fullCount;
    }

    public Double getTotalScore() {
        return totalScore;
    }

    public void setTotalScore(Double totalScore) {
        this.totalScore = totalScore;
    }

    public Integer getQuestionCount() {
        return questionCount;
    }

    public void setQuestionCount(Integer questionCount) {
        this.questionCount = questionCount;
    }

    public Double getExcellentRate() {
        return excellentRate;
    }

    public void setExcellentRate(Double excellentRate) {
        this.excellentRate = excellentRate;
    }

    public Integer getExcellentCount() {
        return excellentCount;
    }

    public void setExcellentCount(Integer excellentCount) {
        this.excellentCount = excellentCount;
    }

    // public void addOptions(String option) {
    // options = StringUtils.trimToNull(options);
    // StringBuilder sb = new StringBuilder(options);
    // if (options != null) {
    // sb.append(DB_ITEM_JOINER);
    // }
    // sb.append(option);
    // this.options = sb.toString();
    // }
    //
    // public List<String> getOptionList() {
    // List<String> list = new LinkedList<String>();
    // options = StringUtils.trimToEmpty(options);
    // String[] values = StringUtils.split(options, DB_ITEM_JOINER);
    // for (String value : values) {
    // list.add(value);
    // }
    // return list;
    // }
}