package cn.com.qmth.stmms.common.utils;

import java.math.BigDecimal;

public class BigDecimalUtils {

    // 榛樿闄ゆ硶杩愮畻绮惧害
    private static final int DEF_DIV_SCALE = 10;

    private BigDecimalUtils() {

    }

    /**
     * 提供精确加法计算的add方法
     * 
     * @param value1
     *            被加数
     * @param value2
     *            加数
     * @return 两个参数的和
     */

    public static double add(double v1, double v2) {

        BigDecimal b1 = new BigDecimal(Double.toString(v1));

        BigDecimal b2 = new BigDecimal(Double.toString(v2));

        return b1.add(b2).doubleValue();

    }

    /**
     * 提供精确减法运算的sub方法
     * 
     * @param value1
     *            被减数
     * @param value2
     *            减数
     * @return 两个参数的差
     */
    public static double sub(double v1, double v2) {

        BigDecimal b1 = new BigDecimal(Double.toString(v1));

        BigDecimal b2 = new BigDecimal(Double.toString(v2));

        return b1.subtract(b2).doubleValue();

    }

    /**
     * 提供精确乘法运算的mul方法
     * 
     * @param value1
     *            被乘数
     * @param value2
     *            乘数
     * @return 两个参数的积
     */
    public static double mul(double v1, double v2) {

        BigDecimal b1 = new BigDecimal(Double.toString(v1));

        BigDecimal b2 = new BigDecimal(Double.toString(v2));

        return b1.multiply(b2).doubleValue();

    }

    /**
     * 鎻愪緵绮剧‘鐨勪箻娉曡繍绠椼�
     * 
     * @param v1
     *            琚箻鏁�
     * @param v2
     *            涔樻暟
     * @return 涓や釜鍙傛暟鐨勭Н
     */
    public static float mul(float v1, float v2) {

        BigDecimal b1 = new BigDecimal(Float.toString(v1));

        BigDecimal b2 = new BigDecimal(Float.toString(v2));

        return b1.multiply(b2).floatValue();

    }

    /**
     * 提供精确的除法运算方法div
     * 
     * @param value1
     *            被除数
     * @param value2
     *            除数
     * @param scale
     *            精确范围
     * @return 两个参数的商
     * @throws IllegalAccessException
     */
    public static double div(double v1, double v2) {

        return div(v1, v2, DEF_DIV_SCALE);

    }

    /**
     * 提供精确的除法运算方法div
     * 
     * @param value1
     *            被除数
     * @param value2
     *            除数
     * @param scale
     *            精确范围
     * @return 两个参数的商
     * @throws IllegalAccessException
     */

    public static double div(double v1, double v2, int scale) {

        if (scale < 0) {

            throw new IllegalArgumentException(

            "The scale must be a positive integer or zero");

        }

        BigDecimal b1 = new BigDecimal(Double.toString(v1));

        BigDecimal b2 = new BigDecimal(Double.toString(v2));

        return b1.divide(b2, scale, BigDecimal.ROUND_HALF_UP).doubleValue();

    }

    /**
     * 鎻愪緵绮剧‘鐨勫皬鏁颁綅鍥涜垗浜斿叆澶勭悊銆�
     * 
     * @param v
     *            闇�鍥涜垗浜斿叆鐨勬暟瀛�
     * @param scale
     *            灏忔暟鐐瑰悗淇濈暀鍑犱綅
     * @return 鍥涜垗浜斿叆鍚庣殑缁撴灉
     */
    public static double round(double v, int scale) {

        if (scale < 0) {

            throw new IllegalArgumentException(

            "The scale must be a positive integer or zero");

        }

        BigDecimal b = new BigDecimal(Double.toString(v));

        BigDecimal one = new BigDecimal("1");

        return b.divide(one, scale, BigDecimal.ROUND_HALF_UP).doubleValue();

    }
}
